/*
 * InterationEventLocatorImpl
 * Version 1.0
 */
package EventEvaluator.Interation;

import java.util.*;
import java.sql.*;
import org.omg.PortableServer.*;

/** 
 * Implements the servant locator for interation event objects.
 * This class implements the servant locator for the interation event objects. It
 * uses the Evictor pattern described at:
 * <p>
 * Advanced CORBA Programming with C++<li>
 * Michi Henning, Steve Vinoski<li>
 * The Addison-Wesley Professional Computing Series
 * <p>
 * This pattern controls the amount of interation event objects resident
 * on the main memory. It maintains an active object map that holds a list
 * of all interation event objects currently on memory and an evictor queue
 * that is updated when there is a reference for an interation event object. The
 * object referenced is placed at the end of the evictor queue. The max limit
 * of elements for this queue determines how many interation event objects can
 * reside at the same time on memory. If the limit is reached and there is a
 * new reference for an interation event not on memory, the first interation event
 * of the evictor queue is removed from there.
 * 
 * @author Francisco Silva
 * @version 1.0
 */
public class InterationEventLocatorImpl extends ServantLocatorPOA
{
    private Hashtable aom;  // active object map (eid, InterationEventImpl)
    private Vector eq;      // evictor queue
    private int MAX = 10;   // max number of elements in evictor queue 
    private org.omg.CORBA.ORB orb;
    private Persistence pobj;

    /**
     * Creates the active object map and the evictor queue.
     * @param orb reference to the orb
     * @param pobj reference to the Persistence object
     */
    public InterationEventLocatorImpl(org.omg.CORBA.ORB orb, Persistence pobj)
    {
	aom = new Hashtable();
	eq  = new Vector(MAX);
	this.orb = orb;
	this.pobj = pobj;
	// System.out.println("====>  Locator Created  !!!!     <=====");
    }

    /**
     * Activate the interation event being requested.
     * This method activates the interation event requested by the user based
     * on the evictor pattern. It receives the key for the interation event that
     * represents the <code>eid</code> (event identification). It then checks if
     * this object exist by calling the method <code>REExist</code> of the
     * <code>Persistence</code> object. If the object exists it checks if it is
     * already on memory by searching the active object map. If it is not on memory
     * it instantiates a new servant for the interation event.
     * @param oid the identification of the interation event being requested
     * @param adapter
     * @param operation the operation on the interation event being requested
     * @param cookie
     */
    public org.omg.PortableServer.Servant preinvoke(byte[] oid, 
	   org.omg.PortableServer.POA adapter, java.lang.String operation, 
	   org.omg.PortableServer.ServantLocatorPackage.CookieHolder cookie)
	throws ForwardRequest
    {
	Servant servant;
	// System.out.println("====>  Begining preinvoke !!!!  <=====");

	// converte o oid em uma string code
	String eid = new String(oid);

	// verifica se o objeto existe
	if (pobj.IEExist(eid)==false)
	    throw new org.omg.CORBA.OBJECT_NOT_EXIST();

	// procura no map para verificar se ja existe
	// um servernte em memoria do objeto
	InterationEventImpl servente = (InterationEventImpl) aom.get(eid);
	if (servente != null) 
	    {
		// servente ja existe. Move este eid para o final da fila
		eq.removeElement(eid);
		eq.addElement(eid);
	    }
	else
	    {
		// servente nao existe. Verifica se a fila esta cheia
		if (eq.size() >= MAX)
		    {
			// a fila esta cheia, remove o elemento do inicio da fila
			String eid_removido = (String) eq.firstElement();
			eq.removeElement(eid_removido);

			// remove o elemento da tabela e realiza um delete no servente
			InterationEventImpl servente_removido 
			    = (InterationEventImpl) aom.get(eid_removido);
			aom.remove(eid_removido);
			servente_removido = null;
		    }

		// a fila nao estava cheia...
		// instancia um novo servente
		servente = new InterationEventImpl(eid, orb, pobj);	

		// insere o code/servente no mapa de objetos ativos
		aom.put(eid, servente);

		// insere o metid na fila evictor
		eq.addElement(eid);
		// novo servente instanciado e inserido no mapa e na fila
	    }

	// impressao da fila: utilizado para Debug
	// Enumeration e = eq.elements(); 
	// System.out.print("Fila Locator: ");
	// for (;e.hasMoreElements();)
	//  System.out.print((String)e.nextElement()+"  ");
	// System.out.println();
	
	servant = servente;
	return servant;
    }
    
    /**
     * Does nothing.
     */
    public void postinvoke
	(byte[] oid, 
	 POA adapter, 
	 String operation, 
	 java.lang.Object cookie, 
	 Servant servant)
    {
    }

    /**
     * Remove from memory the servant of interation event identified by eid.
     * @param eid the identity of the interation event servant to be removed from memory
     */
    public void free (String eid) {
	InterationEventImpl servente = (InterationEventImpl) aom.remove(eid);
	servente = null;
	eq.removeElement(eid);
    } 
    
} // end class InterationEventLocatorImpl
