/*
 * Persistence
 * Version 1.0
 */
package EventEvaluator.Interation;

import java.util.*;
import java.sql.*;
import java.io.*;
import java.lang.*;

/** 
 * Encapsulates all the code responsible for saving the state of 
 * resource event objects to persistent storage. 
 * <p>
 * This code is based on mySQL relational database and uses JDBC
 * to access the database. 
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class Persistence {

    Connection cx = null;
    String database_machine;

    /**
     * Ask for connection with <code>adapatacao</code> database.
     * Receives the name of the machine where mySQL server must be
     * running and then calls the method responsible for connecting
     * with the database <code>adaptacao</code> that stores the state of interation 
     * event objects
     * @param dbmachine the name of the machine where database <code>adaptacao</code> 
     * resides
     */
    public Persistence (String dbmachine) {
	this.database_machine = dbmachine;
	this.connect();
    }

    /**
     * Establish connection with database <code>adaptacao</code>.
     * Database <code>adaptacao</code> must reside on the machine specified
     * by <code>dbmachine</code> variable. This database must have a defined user
     * called <code>fssilva</code>, password <code>AdAptA</code> with
     * access rights to READ, UPDATE, DELETE and ADD tuples to the database tables.
     */
    public void connect () {    
	System.out.println("loading mysql driver...");
	try {
	    Class.forName("org.gjt.mm.mysql.Driver");
	}
	catch (Exception e) {System.out.println(e);}	
	
	String url = "jdbc:mysql://" + this.database_machine + "/adaptacao";
	System.out.println("connecting to database: " + url + "...");	

        try {
            cx = DriverManager.getConnection(url,"fssilva","AdAptA");
        }
        catch (SQLException se) 
	    {
		System.out.println("Error connecting to database !");
		System.out.println(se);
		System.exit(1);
	    } 
    }

    /**
     * Saves the state of a interation event object to database <code>adaptacao</code>.
     * The state of interation event objects are saved to a table called 
     * <code>interation_event</code>.
     * @param eid the id of the interation event
     * @param ior the strigified ior of the interation event
     * @param description the string description of the interation event
     * @param frequency
     * @param class_name
     */    
    public void IESaveState (String eid, String ior, String description,
			     int frequency, String class_name) {
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "insert into interation_event values(?, ?, ?, ?, ?)";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, eid);
	    ps.setString(2, ior);
	    ps.setString(3, description);
            ps.setInt(4, frequency);
	    ps.setString(5, class_name);
	    ps.executeUpdate();
	    ps.close();
	}
	catch (Exception e) { e.printStackTrace(); }
    }

    /**
     * Deletes the state of interation event evaluator identified by 
     * <code>eid</code> from
     * persistent storage.
     * @param eid identification of the interation event evaluator to be deleted
     */ 
    public void IEDeleteState (String eid) {
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "delete from interation_event where eid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, eid);
	    ps.execute();
	    ps.close();
	}
	catch (Exception e) { e.printStackTrace(); }
    }

    /**
     * Check if the state of interation event evaluator identified by <code>eid</code>
     * is stored on database <code>adaptacao</code>.
     * @param eid identification of the interation event evaluator
     * @return <code>true</code> if <code>eid</code> is found and 
     * <code>false</code> otherwise.
     */
    public boolean IEExist(String eid) {
	boolean existe = false;
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select count(*) from interation_event where eid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, eid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    rs.next();
	    existe = (rs.getInt(1) >= 1);
	    rs.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	return existe;
    }
    
    /** 
     * Returns the stringified ior of the interation event evaluator 
     * identified by <code>eid</code>.
     * @param eid identification of the interation event evaluator
     * @return the stringified ior of the interation event evaluator
     */
    public String IEGetRef(String eid) {
	String reid, rior = "";
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select ior from interation_event where eid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, eid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    while (rs.next()) {
		rior = rs.getString(1);
	    }
	    rs.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	return rior;
    }

    /** 
     * Returns the  state of the interation event evaluator identified by 
     * <code>eid</code> 
     * from database <code>adaptacao</code>.
     * @param eid identification of the interation event evaluator
     * @return an array of objects containing the state of the event evaluator
     */
    public Object[] IELoadState(String eid) {
	String reid = "", rior = "", rdescription = "", rclass_name = "";
	int rfrequency = 0;
	Object ret[];
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select * from interation_event where eid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, eid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    while (rs.next()) {
		reid = rs.getString(1);
		rior = rs.getString(2);
		rdescription = rs.getString(3);
		rfrequency = rs.getInt(4);
		rclass_name = rs.getString(5);
	    }
	    rs.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	ret = new Object[5];
	ret[0] = reid;
	ret[1] = rior;
	ret[2] = rdescription;
	ret[3] = rclass_name;
	Integer ifrequency = new Integer(rfrequency);
	ret[4] = ifrequency.toString();
	return ret;
    }

} // end class Persistence











