/*
 * Persistence
 * Version 1.0
 */
package EventEvaluator.Resource;

import java.util.*;
import java.sql.*;
import java.io.*;
import java.lang.*;

/** 
 * Encapsulates all the code responsible for saving the state of 
 * resource event objects to persistent storage. 
 * <p>
 * This code is based on mySQL relational database and uses JDBC
 * to access the database. 
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class Persistence {

    Connection cx = null;
    String database_machine;

    /**
     * Ask for connection with <code>adapatacao</code> database.
     * Receives the name of the machine where mySQL server must be
     * running and then calls the method responsible for connecting
     * with the database <code>adaptacao</code> that stores the state of resource 
     * event objects
     * @param dbmachine the name of the machine where database <code>adaptacao</code> 
     * resides
     */
    public Persistence (String dbmachine) {
	this.database_machine = dbmachine;
	this.connect();
    }

    /**
     * Establish connection with database <code>adaptacao</code>.
     * Database <code>adaptacao</code> must reside on the machine specified
     * by <code>dbmachine</code> variable. This database must have a defined user
     * called <code>fssilva</code>, password <code>AdAptA</code> with
     * access rights to READ, UPDATE, DELETE and ADD tuples to the database tables.
     */
    public void connect () {    
	System.out.println("loading mysql driver...");
	try {
	    Class.forName("org.gjt.mm.mysql.Driver");
	}
	catch (Exception e) {System.out.println(e);}	
	
	String url = "jdbc:mysql://" + this.database_machine + "/adaptacao";
	System.out.println("connecting to database: " + url + "...");	

        try {
            cx = DriverManager.getConnection(url,"fssilva","AdAptA");
        }
        catch (SQLException se) 
	    {
		System.out.println("Error connecting to database !");
		System.out.println(se);
		System.exit(1);
	    } 
    }

    /**
     * Saves the state of a resource event object to database <code>adaptacao</code>.
     * The state of resource event objects are saved to a table called 
     * <code>resource_event</code>.
     * @param eid the id of the resource event
     * @param ior the strigified ior of the resource event
     * @param description the string description of the resource event
     * @param expression the expression that defines the resource event
     * @param duration_time the duration time that the expression must stay true in order
     *                      to trigger the event
     * @param ior_metype the stringified ior of the monitored entity type related with 
     * the event
     */    
    public void RESaveState (String eid, String ior, String description,
			     String expression, int duration_time, String ior_metype) {
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "insert into resource_event values(?, ?, ?, ?, ?, ?)";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, eid);
	    ps.setString(2, ior);
	    ps.setString(3, description);
	    ps.setString(4, expression);
            ps.setInt(5, duration_time);
	    ps.setString(6, ior_metype);
	    ps.executeUpdate();
	    ps.close();
	}
	catch (Exception e) { e.printStackTrace(); }
    }

    /**
     * Deletes the state of resource event evaluator identified by <code>eid</code> from
     * persistent storage.
     * @param eid identification of the resource event evaluator to be deleted
     */ 
    public void REDeleteState (String eid) {
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "delete from resource_event where eid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, eid);
	    ps.execute();
	    ps.close();
	}
	catch (Exception e) { e.printStackTrace(); }
    }

    /**
     * Check if the state of resource event evaluator identified by <code>eid</code>
     * is stored on database <code>adaptacao</code>.
     * @param eid identification of the resource event evaluator
     * @return <code>true</code> if <code>eid</code> is found and 
     * <code>false</code> otherwise.
     */
    public boolean REExist(String eid) {
	boolean existe = false;
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select count(*) from resource_event where eid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, eid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    rs.next();
	    existe = (rs.getInt(1) >= 1);
	    rs.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	return existe;
    }
    
    /** 
     * Returns the stringified ior of the resource event evaluator 
     * identified by <code>eid</code>.
     * @param eid identification of the resource event evaluator
     * @return the stringified ior of the resource event evaluator
     */
    public String REGetRef(String eid) {
	String reid, rior = "";
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select ior from resource_event where eid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, eid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    while (rs.next()) {
		rior = rs.getString(1);
	    }
	    rs.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	return rior;
    }

    /** 
     * Returns the  state of the resource event evaluator identified by <code>eid</code> 
     * from database <code>adaptacao</code>.
     * @param eid identification of the resource event evaluator
     * @return an array of objects containing the state of the event evaluator
     */
    public Object[] RELoadState(String eid) {
	String reid = "", rior = "", rdescription = "", rexpression = "", rior_metype = "";
	int rduration_time = 0;
	Object ret[];
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select * from resource_event where eid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, eid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    while (rs.next()) {
		reid = rs.getString(1);
		rior = rs.getString(2);
		rdescription = rs.getString(3);
		rexpression = rs.getString(4);
		rduration_time = rs.getInt(5);
		rior_metype = rs.getString(6);
	    }
	    rs.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	ret = new Object[6];
	ret[0] = reid;
	ret[1] = rior;
	ret[2] = rdescription;
	ret[3] = rexpression;
	Integer iduration_time = new Integer(rduration_time);
	ret[4] = iduration_time.toString();
	ret[5] = rior_metype;
	return ret;
    }

} // end class Persistence











