/*
* ResourceManagerImpl
* Version 1.0
*/

package EventEvaluator.Resource;

import java.util.*;
import java.io.*;
import java.lang.*;

import Utilities.Log;

import org.omg.PortableServer.*;

/**
 * Implements the servant for the EventEvaluator/Resource/ResourceManager interface.
 * This class implements the methods described at the
 * EventEvaluator/Resource/ResourceManager interface.
 * It is responsible for receiving the notification of changes in the parameters
 * beeing monitored in the execution environment (performed by RMO objects) and
 * evaluating the occurence of adaptation events based on that changes.
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class ResourceManagerImpl extends ResourceManagerPOA {

  private org.omg.CORBA.ORB orb;
  private POA the_poa;
  private  ResourceEventRepository re_r;

  private Hashtable TBActiveEvents;
  private Hashtable TBMonitoredParams;
  private Hashtable TBMonitoredEntities;

  private Utilities.Calculadora.Calculadora calc;
  private Notifier notifierThread;
  private TimerList timerList;
  private Timer timerThread;

  private Log logObj;

  // construtor
  /**
   * Initialize the ResourceManager.
   * Creates the following structures:
   * <ul>
   * <li> TBActiveEvents: the events that are active (ie that are beeing
   *      evaluated by the resource manager;
   * <li> TBMonitoredParams: given a parameter identification (pid),
   *      maintains the list of active events whose boolean expression
   *      contais the pid;
   * <li> TBMonitoredEntities: for each entitie beeing monitored by RMO
   *      objects, maintain the list of parameters beeing monitored as well
   *      as their current value.
   * </ul>
   * Also creates auxiliary objects:
   * <ul>
   * <li> Calculadora: the calculator that evaluates the boolean expression;
   * <li> Notifier: thread responsible for notifying the occurence of events;
   * <li> TimerList: maintain the list of events whose boolean expression are
   *      <code>true</code>;
   * <li> RepTimer: every one second verify, using the TimerList, the events
   *      that occured.
   * </ul>
   * @param orb reference to ORB
   * @param the_poa reference to POA
   * @param re_r reference to resource event repository
   */
  public ResourceManagerImpl(org.omg.CORBA.ORB orb, POA the_poa,
                             ResourceEventRepository re_r ) {
    this.orb = orb;
    this.the_poa = the_poa;
    this.re_r = re_r;

    // creating data-structures
    TBActiveEvents = new Hashtable();
    TBMonitoredParams = new Hashtable();
    TBMonitoredEntities = new Hashtable();

    // creating auxiliary objects
    calc = new Utilities.Calculadora.Calculadora();
    notifierThread = new Notifier(orb);
    notifierThread.start();
    timerList = new TimerList(notifierThread);
    timerThread = new Timer(timerList);
    timerThread.start();

    logObj = logObj.Instance();
  }

  /* -----------------------------------------------------------------
  EventEvaluator.Resource.ResourceManager Interface
  ----------------------------------------------------------------- */

  /**
   * Stop evaluating the event identified by <code>eid</code>.
   * @param eid the event identification
   */
  public void unregister(String eid)
          throws EventEvaluator.NoSuchEvent,EventEvaluator.EventNotBeeingEvaluated {
    try {
      System.out.println("Unregistering event "+eid);
      // verificar se o resourceEvent existe
      ResourceEvent re = re_r.get_re_ref (eid);
      //  encontrar em TBActiveEvents o ae correspondente a eid.
      ActiveEvent ae = (ActiveEvent) TBActiveEvents.get(eid);
      if (ae == null)
        throw new EventEvaluator.EventNotBeeingEvaluated(eid);
      // remover de TimerList todas as entradas com o eid fornecido
      timerList.removeAllEid(eid);
      // Para cada parmetro constante da expressao do evento
      //     encontrar em TBMonitoredParams o vetor de eventos correspondente
      //     a este parametro
      //	   encontrar neste vetor a entrada eid e remove-la
      Vector exPid = ae.exPid();
      for (Enumeration exPidList = exPid.elements(); exPidList.hasMoreElements(); ) {
        String pid = (String) exPidList.nextElement();
        Vector eventList = (Vector)TBMonitoredParams.get(pid);
        if (eventList!=null) {
          if (eventList.remove(eid)==false) {
            System.out.print("unregister possible error: ");
            System.out.println("eventList does not contain "+eid);
          }
        }
        else {
          System.out.print("unregister possible error: ");
          System.out.println("eventList for parameter "+pid+" is null ");
        }
      }
      // remover a entrada em TBActiveEvents correspondente a eid.
      TBActiveEvents.remove(eid);
      // fazer ae = null
      ae = null;
    }
    catch (EventEvaluator.NoSuchEvent nse) {
      throw nse;
    }
  }


  /**
   * Suspende the evaluation of the event identified by <code>eid</code>.
   * @param eid the event identification
   */
  public void suspend(String eid)
          throws EventEvaluator.NoSuchEvent,EventEvaluator.EventNotBeeingEvaluated {
    try {
      // verificar se o resourceEvent existe
      ResourceEvent re = re_r.get_re_ref (eid);
      System.out.println("Suspending event "+eid);
      //  encontrar em TBActiveEvents o ae correspondente a eid.
      ActiveEvent ae = (ActiveEvent) TBActiveEvents.get(eid);
      if (ae == null)
        throw new EventEvaluator.EventNotBeeingEvaluated(eid);
      // remover de TimerList todas as entradas com o eid fornecido
      timerList.removeAllEid(eid);
      // chamar ae.suspend()
      ae.suspend();
    }
    catch (EventEvaluator.NoSuchEvent nse) {
      throw nse;
    }
  }

  /**
   * Resume the evaluation of the event identified by <code>eid</code>.
   * @param eid the event identification
   */
  public void resume(String eid)
          throws EventEvaluator.NoSuchEvent,EventEvaluator.EventNotBeeingEvaluated {
    try {
      // verificar se o resourceEvent existe
      ResourceEvent re = re_r.get_re_ref (eid);
      System.out.println("Resuming event "+eid);
      //  encontrar em TBActiveEvents o ae correspondente a eid.
      ActiveEvent ae = (ActiveEvent) TBActiveEvents.get(eid);
      if (ae == null)
        throw new EventEvaluator.EventNotBeeingEvaluated(eid);
      // chamar ae.resume()
      ae.resume();
    }
    catch (EventEvaluator.NoSuchEvent nse) {
      throw nse;
    }
  }

  /**
   * Stop holding the current value of the parameter <code>pid</code> beeing
   * monitored on entity <code>meid</code>.
   * @param meid the entity identification
   * @param pid the parameter identification
   */
  public void rmo_unregister(String meid, String pid) {
    System.out.println("Unregistering OMR "+meid+"."+pid);
    // remover de TimerList todas as entradas contendo este meid onde a expressao
    // do evento contenha pid
    timerList.removeAllMeidPid(meid, pid);
    // Encontrar em TBMonitoredEntities a hashtable correspondente a meid
    Hashtable paramValues = (Hashtable) TBMonitoredEntities.get(meid);
    if (paramValues == null) {
      System.out.print("rmo_unregister possible error: ");
      System.out.println("TBMonitoredEntities does not contain "+meid);
      return;
    }
    // Encontrar na hashtable o ParameterValue correspondente a pid
    ParameterValue pv = (ParameterValue) paramValues.get(pid);
    if (pv == null) {
      System.out.print("rmo_unregister possible error: ");
      System.out.println("hashtable ParameterValue does not contain "+pid);
      return;
    }
    // remover esta entrada da hashtable
    paramValues.remove(pid);
    // fazer este ParameterValue = null
    pv = null;
  }

  /**
   * Suspend holding the current value of the parameter <code>pid</code> beeing
   * monitored on entity <code>meid</code>.
   * @param meid the entity identification
   * @param pid the parameter identification
   */
  public void rmo_suspend(String meid, String pid) {
    System.out.println("Suspending OMR "+meid+"."+pid);
    // remover de TimerList todas as entradas contendo este meid onde a expressao
    // do evento contenha pid
    timerList.removeAllMeidPid(meid, pid);
    // Encontrar em TBMonitoredEntities a hashtable correspondente a meid
    Hashtable paramValues = (Hashtable) TBMonitoredEntities.get(meid);
    if (paramValues == null) {
      System.out.print("rmo_suspend possible error: ");
      System.out.println("TBMonitoredEntities does not contain "+meid);
      return;
    }
    // Encontrar na hashtable o ParameterValue correspondente a pid
    ParameterValue pv = (ParameterValue) paramValues.get(pid);
    if (pv == null) {
      System.out.print("rmo_suspend possible error: ");
      System.out.println("hashtable ParameterValue does not contain "+pid);
      return;
    }
    pv.suspend();
  }

  /**
   * Resume holding the current value of the parameter <code>pid</code> beeing
   * monitored on entity <code>meid</code>.
   * @param meid the entity identification
   * @param pid the parameter identification
   */
  public void rmo_resume(String meid, String pid) {
    System.out.println("Resuming OMR "+meid+"."+pid);
    // Encontrar em TBMonitoredEntities a hashtable correspondente a meid
    Hashtable paramValues = (Hashtable) TBMonitoredEntities.get(meid);
    if (paramValues == null) {
      System.out.print("rmo_resume possible error: ");
      System.out.println("TBMonitoredEntities does not contain "+meid);
      return;
    }
    // Encontrar na hashtable o ParameterValue correspondente a pid
    ParameterValue pv = (ParameterValue) paramValues.get(pid);
    if (pv == null) {
      System.out.print("rmo_resume possible error: ");
      System.out.println("hashtable ParameterValue does not contain "+pid);
      return;
    }
    pv.resume();
  }

  /**
   * Starts evaluating the resource event <code>re</code>.
   * @param re reference to resource event to start beeing evaluated
   */
  public void register(ResourceEvent re) {
    String eid = re.eid();
    System.out.println("Registering resource event "+eid);
    ActiveEvent ae = (ActiveEvent)TBActiveEvents.get(eid);
    if (ae==null) {

      // create an active event object
      ActiveEvent nae = new ActiveEvent(re);

      // insert it into active events hashtable
      TBActiveEvents.put(eid, nae);
      Vector exPid = nae.exPid();

      // for each parameter on exPid, insert it into monitored parameters hashtable
      for (Enumeration exPidList = exPid.elements() ; exPidList.hasMoreElements() ;) {
        String pid = (String) exPidList.nextElement();
        Vector eventList = (Vector)TBMonitoredParams.get(pid);
        if (eventList==null) {
          Vector newEventList = new Vector(1);
          newEventList.add(eid);
          TBMonitoredParams.put(pid, newEventList);
        }
        else {
          eventList.add(eid);
        }
      }
    }
    // this.dump();
  }

  /**
   * Starts holding the current value of the parameter <code>pid</code> beeing
   * monitored on entity <code>meid</code>.
   * @param meid the entity identification
   * @param pid the parameter identification
   */
  public void rmo_register(String strRmo, String meid, String pid) {
    System.out.println("Registering RMO "+meid+"."+pid);
    Hashtable paramValues = (Hashtable) TBMonitoredEntities.get(meid);
    if (paramValues == null) {

      // paramValues Hashtable does not exit. Creating it.
      // int initial_range = rmo.current_range();
      ParameterValue npv = new ParameterValue(pid, -1);
      Hashtable newParamValues = new Hashtable();
      newParamValues.put(pid, npv);
      TBMonitoredEntities.put(meid, newParamValues);
    }
    else {
      ParameterValue pv = (ParameterValue) paramValues.get(pid);
      if (pv == null) {

        // paramValues hastable does exist. Just adding new ParameterValue object.
        // int initial_range = rmo.current_range();
        ParameterValue npv = new ParameterValue(pid, -1);
        paramValues.put(pid, npv);
      }
      else {
        System.out.println ("Possible error: rmo already registred !");
      }
    }
    // this.dump();
  }

  /**
   * Changes the current value of the parameter <code>pid</code> beeing
   * monitored on entity <code>meid</code>.
   * @param meid the entity identification
   * @param pid the parameter identification
   * @param new_range new value of the parameter
   */
  public void change_parameter(String meid, String pid, int new_range) {
    System.out.println("Updating parameter "+pid+" value on "+meid+" to "+new_range);

    if (logObj.logging()) {
      String sts = new String("Updating parameter "+pid+" to "+new_range+
                              " on "+meid+"\n\n");
      logObj.writeStr(sts);
    }

    try{
      update_parameter_value(meid, pid, new_range);
    }
    catch (OMRNotRegisteredException onr) {
      System.out.println(onr.toString());
      return;
    }

    // evaluate all expressions of active events that contains parameter p
    Vector paramEvents = (Vector) TBMonitoredParams.get(pid);
    if (paramEvents==null)
      return;
    for (Enumeration PEList = paramEvents.elements() ; PEList.hasMoreElements() ;) {
      String ae_id = (String) PEList.nextElement();

      // evaluate expression
      try {
        ActiveEvent ae = (ActiveEvent) TBActiveEvents.get(ae_id);
        if (ae.is_suspended())
          System.out.println("Event "+ae_id+" is suspended");
        else {
          boolean eventOccured = evaluate_expression(ae, meid);
          if (eventOccured) {
            System.out.println("Event "+ae_id+" on "+meid+" is true !");
            // insert event/entity on RepTimer List
            timerList.add(ae, meid);
          }
          else {
            System.out.println("Event "+ae_id+" on "+meid+" is false");
            timerList.remove(ae, meid);
          }
        }
      }
      catch (ParameterWithoutValueException pwv) {
        System.out.println("Event "+ae_id+" on "+meid+" could NOT be evaluated:");
        System.out.println("Parameter "+pwv.pid()+" without value!!");
      }
      catch (Utilities.Calculadora.CException ce) {
        System.out.println("Event "+ae_id+" on "+meid+" could NOT be evaluated:");
        System.out.println("Expression Error!!");
      }
      catch (Exception exc) {
        // System.out.println("Event "+ae_id+" on "+meid+" could NOT be evaluated:");
        // System.out.println("Calculator Exception Error!!");
      }
    }
    // this.dump();
  }

  /* -----------------------------------------------------------------
  Auxiliary Methods
  ----------------------------------------------------------------- */

  /**
   * Updates the current value of the parameter <code>pid</code> beeing
   * monitored on entity <code>meid</code>.
   * @param meid the entity identification
   * @param pid the parameter identification
   * @param new_range new value of the parameter
   */
  private void update_parameter_value(String meid, String pid, int new_range)
          throws OMRNotRegisteredException {

    // find the hashtable of parameter values for entity meid
    Hashtable paramValues =
            (Hashtable) TBMonitoredEntities.get(meid);

    if (paramValues==null) {
      throw new OMRNotRegisteredException(meid, pid);
    }
    else {
      // find the ParameterValue object for parameter pid
      ParameterValue pv = (ParameterValue) paramValues.get(pid);

      // update the value of parameter pid for entity meid
      pv.set_range(new_range);
    }
  }

  /**
   * Evaluates the event <code>ae</code> expression on entity <code>meid</code>.
   * @param ae reference for the active event
   * @param meid entity identification
   */
  private boolean evaluate_expression(ActiveEvent ae, String meid)
          throws ParameterWithoutValueException, Utilities.Calculadora.CException,
          Exception {
    boolean eresult = false;
    System.out.println ("Evaluating expression: "+ae.get_expression());

    // get reference for parameter values hastable of entity meid
    Hashtable paramValues =
            (Hashtable) TBMonitoredEntities.get(meid);

    // find current value for each parameter of the expression
    // Vector calcLines will contain the lines that should be submitted to
    // the calculator. Example:
    //     PUSE=3
    //     MUSE=4
    //     PUSE==3 && MUSE==2
    // Vector exPid will contain the parameters that compose the expression.
    // Exemple: givem the expression PUSE==3 && MUSE==2, exPid
    // will contain PUSE, MUSE
    Vector calcLines = new Vector(1);
    Vector exPid = ae.exPid();
    // for each parameter on exPid, get it current value
    for (Enumeration exPidList = exPid.elements() ; exPidList.hasMoreElements() ;) {
      String pid = (String) exPidList.nextElement();
      // get parameter's current value
      ParameterValue pv = (ParameterValue)paramValues.get(pid);
      if ((pv == null)||(pv.is_suspended()))
        throw new ParameterWithoutValueException(pid);
      else {
        Integer prange = new Integer(pv.get_range());
        String line = pv.get_pid() + "=" + prange.toString();
        calcLines.add(line);
        // System.out.println ("New calcLine: "+line);
      }
    }
    // the last line to be submitted to the calculator should
    // be the expression
    calcLines.add(ae.get_expression());
    // System.out.println("Last calcLine: "+ae.get_expression());
    // System.out.println("Calling the calculator...");

    // calls the Calculator
    String calcR;
    try {
      for (int i = 0; i < calcLines.size()-1 ; i++) {
        // System.out.println (">> "+ (String)calcLines.get(i));
        calcR = calc.calcula((String)calcLines.get(i));
        // System.out.println(">> " + calcR);
      }
      calcR = calc.calcula((String)calcLines.lastElement());
      // System.out.println(">> Resultado: " + calcR);
      if (calcR.equals("1.0"))
        eresult = true;
      else
        eresult = false;
    }
            //     Utilities.Calculadora.CException
    catch (Utilities.Calculadora.CException ce) {
      throw ce;
    }
    catch (Exception exc) {
      throw exc;
    }
    return eresult;
  }

  /**
   * Display on screen the current values of all internal tables.
   */
  private void dump () {
    System.out.println("---------------------------------------------------------");
    System.out.println("Structures DUMP");
    System.out.println("---------------------------------------------------------");
    System.out.println ("TBActiveEvents:");
    for (Enumeration e1 = TBActiveEvents.keys() ; e1.hasMoreElements() ;) {
      System.out.println("   "+e1.nextElement());
    }

    System.out.println("TBMonitoredParams:");
    for (Enumeration e2 = TBMonitoredParams.keys() ; e2.hasMoreElements() ;) {
      String pid = (String)e2.nextElement();
      System.out.println("   "+pid);
      Vector v1 = (Vector)TBMonitoredParams.get(pid);
      if (v1!=null) {
        for (Enumeration e3 = v1.elements() ; e3.hasMoreElements() ;) {
          System.out.println("      "+e3.nextElement());
        }
      }
    }

    System.out.println("TBMonitoredEntites:");
    for (Enumeration e4 = TBMonitoredEntities.keys() ; e4.hasMoreElements() ;) {
      String meid = (String)e4.nextElement();
      System.out.println("   "+meid);
      Hashtable h1 = (Hashtable)TBMonitoredEntities.get(meid);
      if (h1!=null) {
        for (Enumeration e5 = h1.keys() ; e5.hasMoreElements() ;) {
          String pid = (String)e5.nextElement();
          ParameterValue pv = (ParameterValue)h1.get(pid);
          if (pv!=null)
            System.out.println("      "+pid+"="+pv.get_range());
        }
      }
    }

    System.out.println("---------------------------------------------------------");
  }

  public EntityParameterRange[] list_me_parameter_range(java.lang.String meid)
  {
    Hashtable h1 = (Hashtable)TBMonitoredEntities.get(meid);
    EntityParameterRange[] ret = {};
    int i = 0;
    if (h1!=null) {
      ret =  new EntityParameterRange[h1.size()];
      for (Enumeration e5 = h1.keys() ; e5.hasMoreElements() ;) {
        String pid = (String)e5.nextElement();
        ParameterValue pv = (ParameterValue)h1.get(pid);
        ret[i] = new EntityParameterRange();
        ret[i].pid =pid;
        if (pv!=null)
          ret[i].range = pv.get_range();
        i++;
      }
    }
    return ret;
  }
} // end class ResourceManagerImpl






