/*
 * TimerList
 * Version 1.0
 */
package EventEvaluator.Resource;

import java.util.*;
import java.io.*;
import java.lang.*;

class ListNode {
    public Date dt;
    public ActiveEvent ae;
    public String meid;
    public ListNode next;
    public ListNode previous;

    public ListNode(Date dt, ActiveEvent ae, String meid) {
	this.dt = dt;
	this.ae = ae;
	this.meid = meid;
	this.next = null;
	this.previous = null;
    }
}

/** 
 * Hold the list of active events whose expression are currently <code>TRUE</code>.
 * The list is implemented as a double-linked list.
 * @author Francisco Silva
 * @version 1.0
 */
public class TimerList {
    private ListNode firstNode;
    private Hashtable rt;
    private Notifier notifierThread;

    /**
     * Receives a reference to the Nofier thread, responsible for notifing the
     * occurence of events.
     * @param nt notifier thread reference
     */
    public TimerList (Notifier nt) {
	firstNode = null;
	rt = new Hashtable();
	this.notifierThread = nt;
    }

    /**
     * Add an active event on the list.
     * @param ae reference to the active event
     * @param meid entity identification where the active event is taken place
     */
    public synchronized void add (ActiveEvent ae, String meid) {
	int duration_time = ae.duration_time();
	Date futureDate = new Date(System.currentTimeMillis()+duration_time*1000);
	ListNode newNode = new ListNode(futureDate, ae, meid);
	// linked list was empty
	if (firstNode == null) {
	    firstNode = newNode;
	}
	else {
	    ListNode previous = null;
	    ListNode current = firstNode;
	    while (current.dt.compareTo(futureDate) <= 0) {
		previous = current;
		current = current.next;
		if (current == null)
		    break;
	    }
	    // new node must be inserted at the front of the list
	    if (previous == null) {
		newNode.next = current;
		firstNode = newNode;
		current.previous = newNode;
	    }
	    // new node must be inserted in the middle or at the end
	    // of the list
	    else {
		newNode.next = current;
		newNode.previous = previous;
		previous.next = newNode;
		// new node must be inserted in the middle of the list
		if (current != null)
		    current.previous = newNode;
	    }
	}
	// updating hashtable rt with a reference for the new
	// node on list
	rt.put(ae.eid()+"."+meid, newNode);
    }

    /**
     * Remove an active event from the list.
     * @param ae reference to the active event
     * @param meid entity identification where the active event is taken place
     */
    public synchronized void remove (ActiveEvent ae, String meid) {
	ListNode delNode = (ListNode) rt.get(ae.eid()+"."+meid);
	if (delNode != null) {
	    // node to be removed is the first node on list
	    if (firstNode == delNode) {
		firstNode = delNode.next;
		if (delNode.next != null)
		    delNode.next.previous = null;
	    }
	    // node to be removed is on the middle or the end of the list
	    else {
		delNode.previous.next = delNode.next;
		// node to be removed is on the middle of the list
		if (delNode.next != null) 
		    delNode.next.previous = delNode.previous;
	    }
	    rt.remove(ae.eid()+"."+meid);
	    delNode = null;
	}
    }

    /**
     * Remove all active events from the list that are holded in an
     * specific entity.
     * @param eid entity identification
     */
    public synchronized void removeAllEid (String eid) {
	if (firstNode == null)
	    return;
	ListNode delNode = null;
	ListNode current = firstNode;
	while (current != null) {
	    if (eid.compareTo(current.ae.eid())==0) {
		delNode = current;
		rt.remove(current.ae.eid()+"."+current.meid);
		// node to be removed is the first node on list
		if (firstNode == delNode) {
		    firstNode = delNode.next;
		    if (delNode.next != null)
			delNode.next.previous = null;
		}
		// node to be removed is on the middle or the end of the list
		else {
		    delNode.previous.next = delNode.next;
		    // node to be removed is on the middle of the list
		    if (delNode.next != null)
			delNode.next.previous = delNode.previous;
		}
		delNode = null;
	    }
	    current = current.next;
	}
    }

    /**
     * Remove all active events from the list whose boolean expression contains an
     * specific parameter and are holded in an specific entity.
     * @param eid entity identification
     * @param pid parameter identification
     */
    public synchronized void removeAllMeidPid (String meid, String pid) {
	if (firstNode == null)
	    return;
	ListNode delNode = null;
	ListNode current = firstNode;
	while (current != null) {
	    if ((meid.compareTo(current.meid)==0) && current.ae.containsPid(pid)) {
		delNode = current;
		rt.remove(current.ae.eid()+"."+current.meid);
		// node to be removed is the first node on list
		if (firstNode == delNode) {
		    firstNode = delNode.next;
		    if (delNode.next != null)
			delNode.next.previous = null;
		}
		// node to be removed is on the middle or the end of the list
		else {
		    delNode.previous.next = delNode.next;
		    // node to be removed is on the middle of the list
		    if (delNode.next != null)
			delNode.next.previous = delNode.previous;
		}
		delNode = null;
	    }
	    current = current.next;
	}
    }
    
    /**
     * Generate a list of all active events that occured.
     * One active event occurs when its boolean expression stays <code>TRUE</code>
     * during the duration time specified on the resource event definition.
     * <p>
     * The list of active events that occured is passed to a notifier thread,
     * responsible for notifing their occurence.
     */
    public synchronized void generateNotifyList () {
	if (firstNode == null)
	    return;
	Date currentDate = new Date(System.currentTimeMillis());
	if (firstNode.dt.compareTo(currentDate) > 0) {
	    currentDate = null;
	    return;
	}
	Vector list = new Vector();
	ListNode current = firstNode;
	ListNode delNode = null;
	while (current.dt.compareTo(currentDate) <= 0) {
	    NotifyElement e = new NotifyElement(current.ae, current.meid);
	    list.add(e);
	    // remove from node from list
	    firstNode = current.next;
	    if (current.next != null)
		current.next.previous = null;
	    delNode = current;
	    // go to next node
	    current = current.next;
	    rt.remove(delNode.ae.eid()+"."+delNode.meid);
	    delNode = null;
	    if (current == null)
		break;
	}
	// call Notifier passing the notify list
	notifierThread.notify(list);
    }

}


