/*
 * CpuMonitor
 * Version 1.0
 */

package Monitor.Resource;

import MonitoredEntities.*;

import java.util.*;
import java.sql.*;
import java.lang.*;
import java.io.*;

/** 
 * Locally monitor the CPU load.
 * This class locally monitors the percentage of CPU usage in the host. The
 * frequency in which this monitoring occurs is holded by a <code>Frequency</code> object.
 * The value of the currently percentage of CPU usage is holded by a 
 * <code>CurrentRange</code> object. The monitoring performed by a 
 * <code>CpuMonitor</code> object can be suspended and
 * resumed. These operations are controlled by a <code>SuspendMonitor</code> object. 
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class CpuMonitor extends Thread {

    private CurrentRange cr;
    private long us, us_low, sy, id;
    private double cpu_utilization;
    private Frequency fobj;
    private SuspendMonitor smonitor;

    public CpuMonitor (CurrentRange cr, Frequency fobj, SuspendMonitor smonitor) {
	this.cr = cr;
	this.fobj = fobj;
	this.smonitor = smonitor;
	this.cpu_utilization = 0;   
    }

    /**
     * Receives the first line of /proc/stat and extracts the <code>us, us_low, sy</code> 
     * and <code>id</code> values.
     * The first line of /proc/stat looks like this:
     * cpu  686763 32699 95533 9791052
     * This method receives a string containing the first line of /proc/stat and
     * extracts the value of <code>us, us_low, sy</code> and <code>id</code> corresponding 
     * to the numbers that appears in the example above.
     * @param linha the string containing the first line of /proc/stat
     */
    private void extraiValores (String linha) { 

	String linha2 = linha.trim();
	int pos1 = linha2.indexOf("  ");
	int pos2 = linha2.indexOf(" ",pos1+2);
	String strus = linha2.substring(pos1+2,pos2);
	Long Lus = new Long(strus);
	us = Lus.longValue();
	pos1 = pos2;
	pos2 = linha2.indexOf(" ",pos1+1);
	String strus_low = linha2.substring(pos1+1,pos2);
	Long Lus_low = new Long(strus_low);
	us_low = Lus_low.longValue();
	pos1 = pos2;
	pos2 = linha2.indexOf(" ",pos1+1);
	String strsy = linha2.substring(pos1+1,pos2);
	Long Lsy = new Long(strsy);
	sy = Lsy.longValue();
        String strid = linha2.substring(pos2+1,linha2.length());
	Long Lid = new Long(strid);
	id = Lid.longValue();
    }

    /**
     * Continuously check and calculate the percentage of CPU use.
     * This method contains an infinite loop that continuously reads the file
     * /proc/stat and passes the first line of that file to the method 
     * <code>extraiValores</code> where
     * the values of <code>us, us_low, sy</code> and <code>id</code> are extracted. 
     * Two reads of /proc/stat
     * are made in an interval of 2 seconds. The percentage of CPU use is then calculated
     * using the values extracted in both reads. The thread then waits for <code>t</code>
     * seconds and then repeats the loop. <code>t</code> is obtained using the method 
     * <code>get</code> 
     * of the <code>Frequency</code> object, 
     * responsible for holding the frequency in which the monitoring process must
     * occur. The frequency is passed by the user, using the method 
     * <code>change_frequency</code>
     * that belongs to the Rmo interface.
     * <p>
     * After calculating the percentage of CPU use, the loop always check if it must
     * suspend the monitoring process by calling the method 
     * <code>is_monitor_suspended</code>
     * that belongs to a <code>SuspendMonitor</code> object. 
     * This method will stop the monitoring thread
     * if the user called the method <code>suspend</code> that belongs to the 
     * Rmo interface. The monitoring thread will be resumed when the user calls 
     * resume, also from the
     * Rmo interface.
     */
    public void run () {
	long total, used;
	do {
	    total=0; used=0;
	    readFile reader = new readFile("/proc/stat");
	    String linha1 = reader.read1line();
	    reader.close();
	    // System.out.println(linha1);
	    this.extraiValores(linha1);
	    // System.out.println(us + "-" + us_low + "-" + sy + "-" + id);
	    total = us + us_low + sy + id;
	    used = us + us_low + sy;
	    try {
		Thread.sleep(2000);
	    }
	    catch (InterruptedException e) {
		System.err.println(e.toString());
	    }
	    readFile reader2 = new readFile("/proc/stat");
	    String linha2 = reader2.read1line();
	    reader2.close();
	    // System.out.println(linha2);
	    this.extraiValores(linha2);
	    // System.out.println(us + "-" + us_low + "-" + sy + "-" + id);
	    // calculate diff
	    total = us + us_low + sy + id - total;
	    used = us + us_low + sy - used;
	    // System.out.println("diff used: " + used + " / diff total: " + total);
	    cpu_utilization = (used*100)/(total);
	    // System.out.println("Cpu use: " + cpu_utilization);
	    // System.out.println("----------------------------");
	    cr.update(cpu_utilization);

	    // check if the monitor must suspend its execution
	    smonitor.is_monitor_suspended();

	    try {
		Thread.sleep(fobj.get()*1000);
	    }
	    catch (InterruptedException e) {
		System.err.println(e.toString());
	    }
	}
	while(true);
    }

} // end class CpuMonitor


