/*
 * MemoryMonitor
 * Version 1.0
 */

package Monitor.Resource;

import MonitoredEntities.*;

import java.util.*;
import java.sql.*;
import java.lang.*;
import java.io.*;

/** 
 * Locally monitor the amount of free memory available.
 * This class locally monitors the amount of free memory available in the host. The
 * frequency in which this monitoring occurs is holded by a <code>Frequency</code> object.
 * The value of the currently percentage of CPU usage is holded by a 
 * <code>CurrentRange</code> object. The monitoring performed by a 
 * <code>MemoryMonitor</code> object can be suspended and
 * resumed. These operations are controlled by a <code>SuspendMonitor</code> object. 
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class MemoryMonitor extends Thread{

    private CurrentRange cr;
    private double free_memory;
    private Frequency fobj;
    private SuspendMonitor smonitor;

    public MemoryMonitor (CurrentRange cr, Frequency fobj, SuspendMonitor smonitor) {
	this.cr = cr;
	this.fobj = fobj;
	this.smonitor = smonitor;
	this.free_memory = 0;   
	System.out.println ("Memory Monitor Created");
    }

    /**
     * Receives the second line of /proc/meminfo and extracts the amount of free memory
     * available. 
     * The two first lines of /proc/meminfo looks like this:
     *         total:    used:    free:  shared: buffers:  cached:
     * Mem:  264462336 186929152 77533184 76767232 48893952 53342208
     *<p>
     * This method receives a string containing the second line of /proc/meminfo and
     * extracts the value corresponding to the amount of free memory available on the
     * host.  
     * @param linha the string containing the second line of /proc/meminfo
     */
    private void extraiValores (String linha) { 
	String linha2 = linha.trim();
	// System.out.println("extraindo valores de: "+linha2);
	int pos1 = linha2.indexOf(" ");
	while (linha2.charAt(pos1)==' ')
	    pos1 = pos1+1;
	int pos2 = linha2.indexOf(" ",pos1+1);
	while (linha2.charAt(pos2)==' ')
	    pos2 = pos2+1;       
	pos1 = pos2;
	pos2 = linha2.indexOf(" ",pos1+1);
	while (linha2.charAt(pos2)==' ')
	    pos2 = pos2+1;       
	pos1 = pos2;
	pos2 = linha2.indexOf(" ",pos1+1);
	String strfm = linha2.substring(pos1,pos2);
	// System.out.println("Strfm: "+strfm);
	Long Lfm = new Long(strfm);
	free_memory = Lfm.longValue();
    }

    /**
     * Continuously check the amount of free memory available.
     * This method contains an infinite loop that continuously reads the file
     * /proc/meminfo and passes the second line of that file to the method 
     * <code>extraiValores</code> where
     * the value of corresponding to the amount of free memory available is extracted. 
     * The thread then waits for <code>t</code>
     * seconds and then repeats the loop. <code>t</code> is obtained using the method 
     * <code>get</code> 
     * of the <code>Frequency</code> object, 
     * responsible for holding the frequency in which the monitoring process must
     * occur. The frequency is passed by the user, using the method 
     * <code>change_frequency</code>
     * that belongs to the Rmo interface.
     * <p>
     * After getting the amount of free memory available, the loop always check if it must
     * suspend the monitoring process by calling the method 
     * <code>is_monitor_suspended</code>
     * that belongs to a <code>SuspendMonitor</code> object. 
     * This method will stop the monitoring thread
     * if the user called the method <code>suspend</code> that belongs to the 
     * Rmo interface. The
     * monitoring thread will be resumed when the user calls resume, also from the
     * Rmo interface.
     */
    public void run () {
	System.out.println ("Memory Monitor Started");
	do {
	    readFile reader = new readFile("/proc/meminfo");
	    String linha1 = reader.read1line();
	    // System.out.println("linha1: "+linha1);
	    String linha2 = reader.read1line();
	    // System.out.println("linha2: "+linha2);
	    reader.close();
	    this.extraiValores(linha2);
	    // System.out.println("Free Memory: " + free_memory);
	    // System.out.println("----------------------------");
	    cr.update(free_memory/1024); // in Kbytes

	    // check if the monitor must suspend its execution
	    smonitor.is_monitor_suspended();

	    try {
		Thread.sleep(fobj.get()*1000);
	    }
	    catch (InterruptedException e) {
		System.err.println(e.toString());
	    }
	}
	while(true);
    }

} // end class MemoryMonitor








