/*
 * RmoImpl
 * Version 1.0
 */

package Monitor.Resource;

import MonitoredEntities.*;

import java.util.*;
import java.sql.*;
import java.lang.*;
import java.io.*;

import org.omg.PortableServer.*;

/** 
 * Implements the servant for the Monitor.Resource/Rmo interface.
 * This class implements the methods described at the Monitor.Resource/Rmo interface.
 * It is also responsible for creating the objects that a resource monitor
 * must have, execpt the thread responsible for the specific
 * parameter beeing monitored. Objects created:
 * <ul>
 * <li><code>Notifier</code>: notifies the event evaluator when the operation range changes
 * <li><code>CurrentRange</code>: object that holds the current range in
 * which the parameter is operating 
 * <li><code>Frequency</code> object that holds the 
 * frequency in which the monitoring process must occur 
 * <li><code>SuspendMonitor</code> object
 * responsible for suspending and resuming the monitoring thread according to
 * users requests.
 *</ul>
 *
 * @author Francisco Silva
 * @version 1.0
 */
public abstract class RmoImpl extends RmoPOA {
    protected org.omg.CORBA.ORB orb;
    protected POA root_poa; 
    protected CurrentRange cr;
    protected Frequency fobj;
    protected SuspendMonitor smonitor;
    protected Notifier notifierThread;
    protected Parameter par;
    protected String pid;
    protected Entity me;
    protected String meid;
    protected EventEvaluator.Resource.ResourceManager rManager;

    /**
     * Creates the threads and objects that compose a resource monitoring object.
     * @param orb reference to the orb
     * @param root_poa reference to the root poa
     * @param me reference to the monitored entity
     * @param par refernce to the parameter being monitored
     * @param rManager reference to the event evaluator
     */
        public RmoImpl(org.omg.CORBA.ORB orb, POA root_poa, Entity me, Parameter par,
		       EventEvaluator.Resource.ResourceManager rManager)
    {
	this.orb = orb;
	this.root_poa = root_poa;
	this.me = me;
	this.meid = me.meid();
	this.par = par;
	this.pid = par.pid();
	this.rManager = rManager;

	// create the thread responsible for notifying the event evaluator
	// when the range in which the parameter is operating changes
	notifierThread = new Notifier(me, par, rManager);
	notifierThread.start();

	// create current range object, responsible for maintaining the value of the
	// current range in which the parameter this object is monitoring is operating
	cr = new CurrentRange(par, notifierThread);

	// create the thread responsible for monitoring the operation of the
	// parameter and the object responsible for suspend and resume the monitoring
	// process
	fobj = new Frequency (2); // initial frequency is 2 seconds
	smonitor = new SuspendMonitor();
    }

    /**
     * Returns a reference to the parameter being monitored.
     * @return reference to the parameter being monitored
     */
    public Parameter parameter() {
	return par;
    }

    /**
     * Returns a reference to the entity being monitored.
     * @return reference to the entity being monitored
     */
    public Entity me() {
	return me;
    }

    /**
     * Returns the frequency is seconds of the monitoring process.
     * @return frequency of the monitoring process
     */
    public int frequency() {
	return fobj.get();
    }

    /**
     * Returns the range in which the parameter being monitored is operating.
     * @return range in which the parameter being monitored is operating
     */
    public int current_range() {
	return cr.get();
    }
	
    /**
     * Suspends the monitoring process.
     */			   
    public void suspend() {
	smonitor.suspend_monitor();
	rManager.rmo_suspend(meid, pid);
    }

    /**
     * Resumes the monitoring process.
     */
    public void resume() {
	smonitor.unsuspend_monitor();
	rManager.rmo_resume(meid, pid);
    }

    /**
     * Changes the frequency used to monitor the parameter.
     * @param new_frequency the new frequency to be used by the monitoring thread
     */
    public void change_frequency(int new_frequency) {
	fobj.set(new_frequency);
    }

    /**
     * Perform the shutdown of the RMO.
     */
    public void shutdown () {
	System.out.println ("Omr is going down. Please wait");
	rManager.rmo_unregister (meid, pid);
	System.out.println ("Bye...");
	System.exit(0);
    }
    
} // end class RmoImpl


