/*
 * EntityLocatorImpl
 * Version 1.0
 */
package MonitoredEntities;

import java.util.*;
import java.sql.*;
import org.omg.PortableServer.*;

/** 
 * Implements the servant locator for entity objects.
 * This class implements the servant locator for the entity objects. It
 * uses the Evictor pattern described at:
 * <p>
 * Advanced CORBA Programming with C++<li>
 * Michi Henning, Steve Vinoski<li>
 * The Addison-Wesley Professional Computing Series
 * <p>
 * This pattern controls the amount of entity objects resident
 * on the main memory. It maintains an active object map that holds a list
 * of all eneity objects currently on memory and an evictor queue
 * that is updated when there is a reference for an entity object. The
 * object referenced is placed at the end of the evictor queue. The max limit
 * of elements for this queue determines how many eneity objects can
 * reside at the same time on memory. If the limit is reached and there is a
 * new reference for an entity not on memory, the first resource event
 * of the evictor queue is removed from there.
 * 
 * @author Francisco Silva
 * @version 1.0
 */
public class EntityLocatorImpl extends ServantLocatorPOA
{
    private Hashtable aom;  // active object map (meid, EntityImpl)
    private Vector eq;      // evictor queue
    private int MAX = 10;   // max number of elements in evictor queue 
    private org.omg.CORBA.ORB orb;
    private Persistence pobj;

    /**
     * Creates the active object map and the evictor queue.
     * @param orb reference to the orb
     * @param pobj reference to the Persistence object
     */
    public EntityLocatorImpl(org.omg.CORBA.ORB orb, Persistence pobj)
    {
	aom = new Hashtable();
	eq  = new Vector(MAX);
	this.orb = orb;
	this.pobj = pobj;
	// System.out.println("====>  Locator Created  !!!!     <=====");
    }

    /**
     * Activate the entity being requested.
     * This method activates the entity requested by the user based
     * on the evictor pattern. It receives the key for the entity that
     * represents the <code>meid</code> (entity identification). It then checks if
     * this object exist by calling the method <code>EExist</code> of the
     * <code>Persistence</code> object. If the object exists it checks if it is
     * already on memory by searching the active object map. If it is not on memory
     * it instantiates a new servant for the entity.
     * @param oid the identification of the entity being requested
     * @param adapter
     * @param operation the operation on the entity being requested
     * @param cookie
     */
    public org.omg.PortableServer.Servant preinvoke(byte[] oid, 
	   org.omg.PortableServer.POA adapter, java.lang.String operation, 
	   org.omg.PortableServer.ServantLocatorPackage.CookieHolder cookie)
	throws ForwardRequest
    {
	Servant servant;
	// System.out.println("====>  Begining preinvoke !!!!  <=====");

	// converte o oid em uma string code
	String meid = new String(oid);

	// verifica se o objeto existe
	if (pobj.EExist(meid)==false)
	    throw new org.omg.CORBA.OBJECT_NOT_EXIST();

	// procura no map para verificar se ja existe
	// um servernte em memoria do objeto
	EntityImpl servente = (EntityImpl) aom.get(meid);
	if (servente != null) 
	    {
		// servente ja existe. Move este metid para o final da fila
		eq.removeElement(meid);
		eq.addElement(meid);
	    }
	else
	    {
		// servente nao existe. Verifica se a fila esta cheia
		if (eq.size() >= MAX)
		    {
			// a fila esta cheia, remove o elemento do inicio da fila
			String meid_removido = (String) eq.firstElement();
			eq.removeElement(meid_removido);

			// remove o elemento da tabela e realiza um delete no servente
			EntityImpl servente_removido 
			    = (EntityImpl) aom.get(meid_removido);
			aom.remove(meid_removido);
			servente_removido = null;
		    }
		// a fila nao estava cheia...
		// instancia um novo servente
		servente = new EntityImpl(meid, orb, pobj);	

		// insere o code/servente no mapa de objetos ativos
		aom.put(meid, servente);

		// insere o metid na fila evictor
		eq.addElement(meid);
		// novo servente instanciado e inserido no mapa e na fila
	    }

	// impressao da fila: utilizado para Debug
	// Enumeration e = eq.elements(); 
	// System.out.print("Fila Locator: ");
	// for (;e.hasMoreElements();)
	//  System.out.print((String)e.nextElement()+"  ");
	// System.out.println();
	
	servant = servente;
	return servant;
    }

    /**
     * Does nothing.
     */    
    public void postinvoke
	(byte[] oid, 
	 POA adapter, 
	 String operation, 
	 java.lang.Object cookie, 
	 Servant servant)
    {
    }

    /**
     * Remove from memory the servant of entity identified by meid.
     * @param meid the identity of the entity servant to be removed from memory
     */
    public void free (String meid) {
	EntityImpl servente = (EntityImpl) aom.remove(meid);
	servente = null;
	eq.removeElement(meid);
    }
    
} // end class EntityLocatorImpl

