/*
 * EntityRepositoryImpl
 * Version 1.0
 */
package MonitoredEntities;

import java.util.*;
import java.sql.*;
import java.lang.*;
import java.io.*;

import org.omg.PortableServer.*;

/** 
 * Implements the MonitoredEntities.EntityRepository interface.
 * 
 *
 * @author Francisco Silva
 * @version 1.0
 */
public class EntityRepositoryImpl extends EntityRepositoryPOA {
    private org.omg.CORBA.ORB orb;
    private POA et_poa; 
    private POA e_poa;
    private POA p_poa;
    private Persistence pobj;
    private EntityTypeLocatorImpl et_locator;
    private EntityLocatorImpl e_locator;
    private ParameterLocatorImpl p_locator;
    
    /**
     * Receive references to orb, poa and <code>Persistence</code> objects.
     * @param orb reference to orb
     * @param the_poa refence to poa
     * @param Pobj reference to Persistence object
     * @param et_locator reference to entity type locator
     * @param e_locator reference to entity locator
     * @param p_locator reference to parameter locator
     */
    public EntityRepositoryImpl(org.omg.CORBA.ORB orb, POA et_poa, 
				POA e_poa, POA p_poa, Persistence pobj,
				EntityTypeLocatorImpl et_locator,
				EntityLocatorImpl e_locator,
				ParameterLocatorImpl p_locator)
    {
	this.orb = orb;
	this.et_poa = et_poa;
	this.e_poa = e_poa;
	this.p_poa = p_poa;
	this.pobj = pobj;
	this.et_locator = et_locator;
	this.e_locator = e_locator;
	this.p_locator = p_locator;	
    }
  
    /** 
     * Creates a entity type object.
     * Creates a entity type object and saves it state to persistent storage.
     * @param metid the identification of the object being created
     * @param description its description 
     * @return the ior reference for the entity type object that was created
     */
    public EntityType add_metype(String metid, String description)
	throws EntityTypeAlreadyExist 
    {
	if (pobj.ETExist(metid))
	  throw new EntityTypeAlreadyExist(metid); 
	// System.out.println("criando tipo entidade " + metid);
	EntityType et = null;
	try
	    {
		org.omg.CORBA.Object obj = et_poa.create_reference_with_id
		    (metid.getBytes(), EntityTypeHelper.id());
		et = EntityTypeHelper.narrow(obj);
	    }
	catch (org.omg.PortableServer.POAPackage.WrongPolicy wp)
	    {
		wp.printStackTrace();
	    }
	String strIOR = orb.object_to_string(et);

	// save object state
	pobj.ETSaveState (metid, strIOR, description);
	System.out.println("entity type " + metid + " created...");
	// System.out.println(strIOR);
	return et;
    }

    /** 
     * Creates a entity object.
     * Creates a entity object and saves it state to persistent storage.
     * @param meid the identification of the object being created
     * @param metype a reference for the entity type of the entity to be created
     * @param description its description 
     * @param attributes its list of attributes
     * @return the ior reference for the entity object that was created
     */
    public Entity add_me(String meid, EntityType metype, String description, 
			 String[] attributes) throws EntityAlreadyExist 
    {
	if (pobj.EExist(meid))
	  throw new EntityAlreadyExist(meid); 
	// System.out.println("criando entidade " + meid);
	Entity entity = null;
	try
	    {
		org.omg.CORBA.Object obj = e_poa.create_reference_with_id
		    (meid.getBytes(), EntityHelper.id());
		entity = EntityHelper.narrow(obj);
	    }
	catch (org.omg.PortableServer.POAPackage.WrongPolicy wp)
	    {
		wp.printStackTrace();
	    }
	String strIOR = orb.object_to_string(entity);

	// save object state
	pobj.ESaveState (meid, strIOR, metype.metid(), description, attributes);
	System.out.println("entity " + meid + " created...");
	// System.out.println(strIOR);
	return entity;
    }
    
    /** 
     * Creates a parameter object.
     * Creates a parameter object and saves it state to persistent storage.
     * @param pid the identification of the object being created
     * @param description its description 
     * @param metype a reference to the entity type the parameter is related to
     * @param range its list of ranges
     * @return the ior reference for the parameter object that was created
     */
    public Parameter add_parameter(String pid, String description, EntityType metype, 
				   range[] ranges) throws ParameterAlreadyExist
    { 
	if (pobj.PExist(pid))
	  throw new ParameterAlreadyExist(pid); 
	// System.out.println("criando parametro " + pid);
	Parameter parameter = null;
	try
	    {
		org.omg.CORBA.Object obj = p_poa.create_reference_with_id
		    (pid.getBytes(), ParameterHelper.id());
		parameter = ParameterHelper.narrow(obj);
	    }
	catch (org.omg.PortableServer.POAPackage.WrongPolicy wp)
	    {
		wp.printStackTrace();
	    }
	String strIOR = orb.object_to_string(parameter);

	// save object state
	pobj.PSaveState (pid, strIOR, description, metype.metid(), ranges);
	System.out.println("parameter " + pid + " created...");
	// System.out.println(strIOR);
	return parameter;
    }

    /**
     * Returns the ior reference for the entity type identified by <code>metid</code>.
     * @param metid the identification of the entity type
     * @return the reference for the entity type
     */
    public EntityType get_metype_ref(String metid) throws NoSuchEntityType {
	if (! pobj.ETExist(metid))
	    throw new NoSuchEntityType(metid); 
	String strIOR = pobj.ETGetRef(metid);
	EntityType et =
	    EntityTypeHelper.narrow(orb.string_to_object(strIOR));
	System.out.println("returning reference to " + metid + "...");
	// System.out.println(strIOR);
	return et;
    }     

    /**
     * Returns the ior reference for the entity identified by <code>meid</code>.
     * @param meid the identification of the entity
     * @return the reference for the entity
     */
    public Entity get_me_ref(String meid) throws NoSuchEntity {
	if (! pobj.EExist(meid))
	    throw new NoSuchEntity(meid); 
	String strIOR = pobj.EGetRef(meid);
	Entity entity =
	    EntityHelper.narrow(orb.string_to_object(strIOR));
	System.out.println("returning reference to " + meid + "...");
	// System.out.println(strIOR);
	return entity;
    }

    /**
     * Returns the ior reference for the parameter identified by <code>pid</code>.
     * @param pid the identification of the parameter
     * @return the reference for the parameter
     */
    public Parameter get_parameter_ref(String pid) throws NoSuchParameter {
	if (! pobj.PExist(pid))
	    throw new NoSuchParameter(pid); 
	String strIOR = pobj.PGetRef(pid);
	Parameter parameter =
	    ParameterHelper.narrow(orb.string_to_object(strIOR));
	System.out.println("returning reference to " + pid + "...");
	// System.out.println(strIOR);
	return parameter;
    }

    /**
     * Destroy an entity type object by removing it from persistent storage.
     * @param metid the identification of the entity type to be destroyed
     */
    public void erase_metype(String metid) 
	throws NoSuchEntityType,RIViolate
    {
	// System.out.println("tentando excluir tipo entidade " + metid + "...");
	if (! pobj.ETExist(metid))
	    throw new NoSuchEntityType(metid);
	if (pobj.ETRIViolate(metid))
	    throw new RIViolate();
	pobj.ETDeleteState (metid);
	et_locator.free(metid);
	System.out.println("entity type " + metid + " deleted...");
    }

    /**
     * Destroy an entity object by removing it from persistent storage.
     * @param meid the identification of the entity to be destroyed
     */
    public void erase_me(String meid) throws NoSuchEntity {
	// System.out.println("tentando excluir entidade " + meid + "...");
	if (! pobj.EExist(meid))
	    throw new NoSuchEntity(meid);
	pobj.EDeleteState (meid);
	e_locator.free(meid);
	System.out.println("entity " + meid + " deleted...");
    }

    /**
     * Destroy an parameter object by removing it from persistent storage.
     * @param pid the identification of the parameter to be destroyed
     */
    public void erase_parameter(String pid) throws NoSuchParameter {
	// System.out.println("tentando excluir parametro " + pid + "...");
	if (! pobj.PExist(pid))
	    throw new NoSuchParameter(pid);
	pobj.PDeleteState (pid);
	p_locator.free(pid);
	System.out.println("parameter " + pid + " deleted...");
    }
    /**
      * List all types in persistent storage
      */
    public String[] list_metype()
    {
      Vector vret = pobj.ETListMetids();
      String[] ret = new String[vret.size()];
      for (int i = 0; i < ret.length; i++)
      {
        ret[i] = (String)vret.get(i);
      }
      return ret;
    }

     /**
      * List all entities objects in persistent storage
      */
     public String[] list_me()
     {
       Vector vret = pobj.EListMeids();
       String[] ret = new String[vret.size()];
       for (int i = 0; i < ret.length; i++)
       {
         ret[i] = (String)vret.get(i);
       }
       return ret;
     }

     /**
      * List all parameters in persistent storage
      */
     public String[] list_parameter()
     {
       Vector vret = pobj.PListPids();
       String[] ret = new String[vret.size()];
       for (int i = 0; i < ret.length; i++)
       {
         ret[i] = (String)vret.get(i);
       }
       return ret;
     }

} // end class EntityRepositoryImpl




