/*
 * ParameterLocatorImpl
 * Version 1.0
 */
package MonitoredEntities;

import java.util.*;
import java.sql.*;
import org.omg.PortableServer.*;

/** 
 * Implements the servant locator for parameter objects.
 * This class implements the servant locator for the parameter objects. It
 * uses the Evictor pattern described at:
 * <p>
 * Advanced CORBA Programming with C++<li>
 * Michi Henning, Steve Vinoski<li>
 * The Addison-Wesley Professional Computing Series
 * <p>
 * This pattern controls the amount of parameter objects resident
 * on the main memory. It maintains an active object map that holds a list
 * of all eneity objects currently on memory and an evictor queue
 * that is updated when there is a reference for an parameter object. The
 * object referenced is placed at the end of the evictor queue. The max limit
 * of elements for this queue determines how many eneity objects can
 * reside at the same time on memory. If the limit is reached and there is a
 * new reference for an parameter not on memory, the first resource event
 * of the evictor queue is removed from there.
 * 
 * @author Francisco Silva
 * @version 1.0
 */
public class ParameterLocatorImpl extends ServantLocatorPOA
{
    private Hashtable aom;  // active object map (pid, Parameter)
    private Vector eq;      // evictor queue
    private int MAX = 10;   // max number of elements in evictor queue 
    private org.omg.CORBA.ORB orb;
    private Persistence pobj;

    /**
     * Creates the active object map and the evictor queue.
     * @param orb reference to orb
     * @param pobj reference to Persistence object
     */
    public ParameterLocatorImpl(org.omg.CORBA.ORB orb, Persistence pobj)
    {
	aom = new Hashtable();
	eq  = new Vector(MAX);
	this.orb = orb;
	this.pobj = pobj;
	// System.out.println("====>  Locator Created  !!!!     <=====");
    }

    /**
     * Activate the parameter being requested.
     * This method activates the parameter requested by the user based
     * on the evictor pattern. It receives the key for the parameter that
     * represents the <code>meid</code> (parameter identification). It then checks if
     * this object exist by calling the method <code>EExist</code> of the
     * <code>Persistence</code> object. If the object exists it checks if it is
     * already on memory by searching the active object map. If it is not on memory
     * it instantiates a new servant for the parameter.
     * @param oid the identification of the parameter being requested
     * @param adapter
     * @param operation the operation on the parameter being requested
     * @param cookie
     */
    public org.omg.PortableServer.Servant preinvoke(byte[] oid, 
	   org.omg.PortableServer.POA adapter, java.lang.String operation, 
	   org.omg.PortableServer.ServantLocatorPackage.CookieHolder cookie)
	throws ForwardRequest
    {
	Servant servant;
	// System.out.println("====>  Begining preinvoke !!!!  <=====");

	// converte o oid em uma string code
	String pid = new String(oid);

	// verifica se o objeto existe
	if (pobj.PExist(pid)==false)
	    throw new org.omg.CORBA.OBJECT_NOT_EXIST();

	// procura no map para verificar se ja existe
	// um servernte em memoria do objeto
	ParameterImpl servente = (ParameterImpl) aom.get(pid);
	if (servente != null) 
	    {
		// servente ja existe. Move este metid para o final da fila
		eq.removeElement(pid);
		eq.addElement(pid);
	    }
	else
	    {
		// servente nao existe. Verifica se a fila esta cheia
		if (eq.size() >= MAX)
		    {
			// a fila esta cheia, remove o elemento do inicio da fila
			String pid_removido = (String) eq.firstElement();
			eq.removeElement(pid_removido);

			// remove o elemento da tabela e realiza um delete no servente
			ParameterImpl servente_removido 
			    = (ParameterImpl) aom.get(pid_removido);
			aom.remove(pid_removido);
			servente_removido = null;
		    }
		// a fila nao estava cheia...
		// instancia um novo servente
		servente = new ParameterImpl(pid, orb, pobj);	

		// insere o code/servente no mapa de objetos ativos
		aom.put(pid, servente);

		// insere o metid na fila evictor
		eq.addElement(pid);
		// novo servente instanciado e inserido no mapa e na fila
	    }

	// impressao da fila: utilizado para Debug
	// Enumeration e = eq.elements(); 
	// System.out.print("Fila Locator: ");
	// for (;e.hasMoreElements();)
	//  System.out.print((String)e.nextElement()+"  ");
	// System.out.println();
	
	servant = servente;
	return servant;
    }
    
    /**
     * Does nothing.
     */    
    public void postinvoke
	(byte[] oid, 
	 POA adapter, 
	 String operation, 
	 java.lang.Object cookie, 
	 Servant servant)
    {
    }

    /**
     * Remove from memory the servant of the parameter identified by meid.
     * @param pid the identity of the parameter whose servant is to be removed from memory
     */  
    public void free (String pid) {
	ParameterImpl servente = (ParameterImpl) aom.remove(pid);
	servente = null;
	eq.removeElement(pid);
    }
  
} // end class ParameterLocatorImpl

