/*
 * Persistence
 * Version 1.0
 */
package MonitoredEntities;

import java.util.*;
import java.sql.*;
import java.io.*;
import java.lang.*;

/**
 * Store and restore the state of entity, entity type and parameter objects on
 * persistent storage.
 * Encapsultes all the code necessary for storing and restoring the state
 * of entity, entity type and parameters objects on persistent storage. This
 * version uses jdbc to connect to mm, a driver to my_sql database.
 * @author Francisco Silva
 * @version 1.0
 */
public class Persistence {

    Connection cx = null;
    String database_machine;


    /**
     * Store the machine where the database must be running and conects to it.
     * Receives the name of the machine where mySQL server must be
     * running and then calls the method responsible for connecting
     * with the database <code>adaptacao</code> that stores the state of resource 
     * event objects
     * @param dbmachine the name of the machine where database <code>adaptacao</code> 
     * resides
     */
    public Persistence (String dbmachine) {
	this.database_machine = dbmachine;
	this.connect();
    }

    /**
     * Establish connection with database <code>adaptacao</code>.
     * Database <code>adaptacao</code> must reside on the machine specified
     * by <code>dbmachine</code> variable. This database must have a defined user
     * called <code>fssilva</code>, password <code>AdAptA</code> with
     * access rights to READ, UPDATE, DELETE and ADD tuples to the database tables.
     */
    public void connect () {    
	System.out.println("loading mysql driver...");
	try {
	    Class.forName("org.gjt.mm.mysql.Driver");
	}
	catch (Exception e) {System.out.println(e);}	
	
	String url = "jdbc:mysql://" + this.database_machine + "/adaptacao";
	System.out.println("connecting to database: " + url + "...");	

        try {
            cx = DriverManager.getConnection(url,"fssilva","AdAptA");
        }
        catch (SQLException se) 
	    {
		System.out.println("Error connecting to database !");
		System.out.println(se);
		System.exit(1);
	    } 
    }
    
    /**
     * Saves the state of a entity type object to database <code>adaptacao</code>.
     * The state of entity type objects are saved to a table called 
     * <code>entity_type</code>.
     */
    public void ETSaveState (String metid, String ior, String description) {
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "insert into entity_type values(?, ?, ?)";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, metid);
	    ps.setString(2, ior);
	    ps.setString(3, description);
	    ps.executeUpdate();
	    ps.close();
	}
	catch (Exception e) { e.printStackTrace(); }
    }

    /**
     * Checks if there is entity or parameter objects related to entity type 
     * <code>metid</code>.
     * This method is used to verify if there will be an relational integrity violation
     * if entity type identified by <code>metid</code> is deleted. This is done
     * by verifying if there is entity or parameter objects related to entity type 
     * <code>meid</code>.
     * @param metid the identification of the entity type object
     * @return <code>true</code>, if there is entity or parameter objects related 
     * to entity type <code>metid</code>, <code>false</code> otherwise.
     */
    public boolean ETRIViolate(String metid) {
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select count(*) from entity where metid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, metid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    rs.next();
	    if ((rs.getInt(1) >= 1)) {
		rs.close();
		return true;
	    }
	    rs.close();
	    
	    String s2 = "select count(*) from parameter where metid=?";
	    java.sql.PreparedStatement ps2 = cx.prepareStatement(s2);
	    ps2.setString(1, metid);
	    java.sql.ResultSet rs2 = ps2.executeQuery();
	    rs2.next();
	    if ((rs2.getInt(1) >= 1)) {
		rs2.close();
		return true;
	    }
	    rs2.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	return false;
    }

    /**
     * Deletes the state of entity type object identified by <code>metid</code> from
     * persistent storage.
     * @param metid identification of the entity type object to be deleted
     */ 
    public void ETDeleteState (String metid) {
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "delete from entity_type where metid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, metid);
	    ps.execute();
	    ps.close();
	}
	catch (Exception e) { e.printStackTrace(); }
    }

    /**
     * Check if the state of entity type identified by <code>metid</code>
     * is stored on database <code>adaptacao</code>.
     * @param metid identification of the entity type
     * @return <code>true</code> if <code>metid</code> is found and 
     * <code>false</code> otherwise.
     */
    public boolean ETExist(String metid) {
	boolean existe = false;
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select count(*) from entity_type where metid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, metid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    rs.next();
	    existe = (rs.getInt(1) >= 1);
	    rs.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	return existe;
    }

    /** 
     * Returns the stringified ior of the entity type 
     * identified by <code>metid</code>.
     * @param metid identification of the entity type
     * @return the stringified ior of the entity type
     */    
    public String ETGetRef(String metid) {
	String rmetid, rior = "", rdescription;
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select * from entity_type where metid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, metid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    while (rs.next()) {
		rmetid = rs.getString(1);
		rior = rs.getString(2);
		rdescription = rs.getString(3);
	    }
	    rs.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	return rior;
    }

    /** 
     * Returns the  state of the entity type identified by <code>metid</code> 
     * from database <code>adaptacao</code>.
     * @param metid identification of the entity type
     * @return an array of objects containing the state of the entity type
     */
    public Object[] ETLoadState(String metid) {
	String rmetid = "", rior = "", rdescription = ""; 
	Object ret[];
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select * from entity_type where metid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, metid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    while (rs.next()) {
		rmetid = rs.getString(1);
		rior = rs.getString(2);
		rdescription = rs.getString(3);
	    }
	    rs.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	ret = new Object[3];
	ret[0] = rmetid;
	ret[1] = rior;
	ret[2] = rdescription;
	return ret;
    }

    /**
     * Saves the state of a entity object to database <code>adaptacao</code>.
     * The state of entity objects are saved to a table called 
     * <code>entity</code>.
     */
    public void ESaveState (String meid, String ior, String metid, String description,
			    String[] attributes) {
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "insert into entity values(?, ?, ?, ?)";
	    // System.out.println("salvando no banco de dados entidade " + meid);
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, meid);
	    ps.setString(2, ior);
	    ps.setString(3, metid);
	    ps.setString(4, description);
	    ps.executeUpdate();
	    // System.out.println("salvando no banco de dados a lista de atributos");
	    if (attributes.length != 0)
		for (int i=0; i < attributes.length; i++) {
		    String sp = "insert into entity_attributes values(?, ?)";
		    java.sql.PreparedStatement pssp = cx.prepareStatement(sp);
		    pssp.setString(1, meid);
		    pssp.setString(2, attributes[i]);
		    pssp.executeUpdate();
		    pssp.close();
		}
	    ps.close();
	}
	catch (Exception e) { 
	    System.out.println ("Erro em ESaveState !!!");
	    e.printStackTrace(); }
    }

    /**
     * Deletes the state of entity object identified by <code>meid</code> from
     * persistent storage.
     * @param meid identification of the entity object to be deleted
     */     
    public void EDeleteState (String meid) {
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "delete from entity_attributes where meid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, meid);
	    ps.execute();
	    ps.close();
	    String s2 = "delete from entity where meid=?";
	    java.sql.PreparedStatement ps2 = cx.prepareStatement(s2);
	    ps2.setString(1, meid);
	    ps2.execute();
	    ps2.close();
	}
	catch (Exception e) { e.printStackTrace(); }
    }

    /**
     * Check if the state of entity identified by <code>meid</code>
     * is stored on database <code>adaptacao</code>.
     * @param meid identification of the entity
     * @return <code>true</code> if <code>meid</code> is found and 
     * <code>false</code> otherwise.
     */
    public boolean EExist(String meid) {
	boolean existe = false;
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select count(*) from entity where meid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, meid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    rs.next();
	    existe = (rs.getInt(1) >= 1);
	    rs.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	return existe;
    }

    /** 
     * Returns the stringified ior of the entity 
     * identified by <code>meid</code>.
     * @param meid identification of the entity
     * @return the stringified ior of the entity
     */     
    public String EGetRef(String meid) {
	String rmeid, rior = "", rmetid, rdescription;
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select * from entity where meid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, meid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    while (rs.next()) {
		rmeid = rs.getString(1);
		rior = rs.getString(2);
		rmetid = rs.getString(3);
		rdescription = rs.getString(4);
	    }
	    rs.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	return rior;
    }

    /** 
     * Returns the  state of the entity identified by <code>meid</code> 
     * from database <code>adaptacao</code>.
     * @param meid identification of the entity
     * @return an vector of objects containing the state of the entity
     */
    public Vector ELoadState(String meid) {
	String rmeid = "", rior = "", rmetid = "", rdescription = ""; 
	Vector ret = new Vector();
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select * from entity where meid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, meid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    while (rs.next()) {
		rmeid = rs.getString(1);
		rior = rs.getString(2);
		rmetid = rs.getString(3);
		rdescription = rs.getString(4);
	    }
	    rs.close();
	    ret.addElement(rmeid);
	    ret.addElement(rior);
	    ret.addElement(rmetid);
	    ret.addElement(rdescription);
	    
	    // reading attribute list from database
	    String s2 = "select attribute from entity_attributes where meid=?";
	    java.sql.PreparedStatement ps2 = cx.prepareStatement(s2);
	    ps2.setString(1, meid);
	    java.sql.ResultSet rs2 = ps2.executeQuery();
	    while (rs2.next())
		ret.addElement(rs2.getString(1));
	    rs2.close();
	}
	catch (Exception e) { e.printStackTrace(); }	
	return ret;
    }

    /**
     * Saves the state of a parameter object to database <code>adaptacao</code>.
     * The state of parameter objects are saved to a table called 
     * <code>parameter</code>.
     */
    public void PSaveState (String pid, String ior, String metid, String description,
			    range[] range_list) {
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "insert into parameter values(?, ?, ?, ?)";
	    // System.out.println("salvando no banco de dados entidade " + pid);
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, pid);
	    ps.setString(2, ior);
	    ps.setString(3, description);
	    ps.setString(4, metid);
	    ps.executeUpdate();
	    // System.out.println("salvando no banco de dados as faixas de valores");
	    if (range_list.length != 0)
		for (int i=0; i < range_list.length; i++) {
		    String sp = "insert into parameter_ranges values(?, ?, ?, ?)";
		    java.sql.PreparedStatement pssp = cx.prepareStatement(sp);
		    pssp.setString(1, pid);
		    pssp.setInt(2, i);
		    pssp.setDouble(3, range_list[i].initial_value);
		    pssp.setDouble(4, range_list[i].final_value);
		    pssp.executeUpdate();
		    pssp.close();
		}
	    ps.close();
	}
	catch (Exception e) { 
	    System.out.println ("Erro em PSaveState !!!");
	    e.printStackTrace(); }
    }

    /**
     * Deletes the state of parameter object identified by <code>pid</code> from
     * persistent storage.
     * @param pid identification of the parameter object to be deleted
     */ 
    public void PDeleteState (String pid) {
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "delete from parameter_ranges where pid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, pid);
	    ps.execute();
	    ps.close();
	    String s2 = "delete from parameter where pid=?";
	    java.sql.PreparedStatement ps2 = cx.prepareStatement(s2);
	    ps2.setString(1, pid);
	    ps2.execute();
	    ps2.close();
	}
	catch (Exception e) { e.printStackTrace(); }
    }

    /**
     * Check if the state of the parameter identified by <code>pid</code>
     * is stored on database <code>adaptacao</code>.
     * @param pid identification of the parameter
     * @return <code>true</code> if <code>pid</code> is found and 
     * <code>false</code> otherwise.
     */
    public boolean PExist(String pid) {
	boolean existe = false;
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select count(*) from parameter where pid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, pid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    rs.next();
	    existe = (rs.getInt(1) >= 1);
	    rs.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	return existe;
    }

    /** 
     * Returns the stringified ior of the parameter 
     * identified by <code>pid</code>.
     * @param pid identification of the parameter
     * @return the stringified ior of the parameter
     */     
    public String PGetRef(String pid) {
	String rpid, rior = "", rmetid, rdescription;
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select * from parameter where pid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, pid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    while (rs.next()) {
		rpid = rs.getString(1);
		rior = rs.getString(2);
		rdescription = rs.getString(3);
		rmetid = rs.getString(4);
	    }
	    rs.close();
	}
	catch (Exception e) { e.printStackTrace(); }
	return rior;
    }

    /** 
     * Returns the  state of the parameter identified by <code>pid</code> 
     * from database <code>adaptacao</code>.
     * @param pid identification of the parameter
     * @return an vector of objects containing the state of the parameter
     */
    public Vector PLoadState(String pid) {
	String rpid = "", rior = "", rmetid = "", rdescription = ""; 
	Vector ret = new Vector();
	try {
	    if (cx.isClosed())
		this.connect();
	    String s = "select * from parameter where pid=?";
	    java.sql.PreparedStatement ps = cx.prepareStatement(s);
	    ps.setString(1, pid);
	    java.sql.ResultSet rs = ps.executeQuery();
	    while (rs.next()) {
		rpid = rs.getString(1);
		rior = rs.getString(2);
		rdescription = rs.getString(3);
		rmetid = rs.getString(4);
	    }
	    rs.close();
	    ret.addElement(rpid);
	    ret.addElement(rior);
	    ret.addElement(rdescription);
	    ret.addElement(rmetid);
	    
	    // reading attribute list from database
	    String s2 = "select initial, final  from parameter_ranges where pid=? order by seq";
	    java.sql.PreparedStatement ps2 = cx.prepareStatement(s2);
	    ps2.setString(1, pid);
	    java.sql.ResultSet rs2 = ps2.executeQuery();
	    double rinitial = 0, rfinal = 0;
	    while (rs2.next()) {
	      rinitial = rs2.getDouble(1);
	      ret.addElement(Double.toString(rinitial));
	      rfinal = rs2.getDouble(2);
	      ret.addElement(Double.toString(rfinal));
	    }
	    rs2.close();
	}
	catch (Exception e) { e.printStackTrace(); }	
	return ret;
    }
 /**
     * Returns the  list of the <code>pid</code>
     * from database <code>adaptacao</code>.
     */
    public Vector PListPids() {
        Vector ret = new Vector();
        try {
            if (cx.isClosed())
                this.connect();
            String s = "select pid from parameter";
            java.sql.PreparedStatement ps = cx.prepareStatement(s);
            java.sql.ResultSet rs = ps.executeQuery();
            while (rs.next()) {
                String rpid = rs.getString(1);
                ret.addElement(rpid);
            }
            rs.close();
        }
        catch (Exception e) { e.printStackTrace(); }
        return ret;
    }

    /**
     * Returns the  list of the <code>meid</code>
     * from database <code>adaptacao</code>.
     */
    public Vector EListMeids() {
        Vector ret = new Vector();
        try {
            if (cx.isClosed())
                this.connect();
            String s = "select meid from entity";
            java.sql.PreparedStatement ps = cx.prepareStatement(s);
            java.sql.ResultSet rs = ps.executeQuery();
            while (rs.next()) {
                String rmeid = rs.getString(1);
                ret.addElement(rmeid);
            }
            rs.close();
        }
        catch (Exception e) { e.printStackTrace(); }
        return ret;
    }


    /**
     * Returns the  list of the <code>metid</code>
     * from database <code>adaptacao</code>.
     */
    public Vector ETListMetids() {
        Vector ret = new Vector();
        try {
            if (cx.isClosed())
                this.connect();
            String s = "select metid from entity_type";
            java.sql.PreparedStatement ps = cx.prepareStatement(s);
            java.sql.ResultSet rs = ps.executeQuery();
            while (rs.next()) {
                String metid = rs.getString(1);
                ret.addElement(metid);
            }
            rs.close();
        }
        catch (Exception e) { e.printStackTrace(); }
        return ret;
    }
} // end class Persistence











